/* 
 * David Rieman

 * Section VA
 * CSE142, Spring 1999
 * Homework 4, part B
 * Traffic animation
 *
 *  Program to graphically control vehicle(s) in environment.
 */


#include "gp142.h"
#include <stdio.h>
#include <math.h>

#include <time.h>

/* Logical constants */
#define TRUE    		1
#define FALSE   		0

/* 
 * For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.  The
 * constant FILL is used in function calls when a solid
 * shape is desired.
 */
#define FILL 0

#define ASCII_DEC_MOD	48

/* Points achieved during gameplay */
#define POINTS_CAR		25
#define POINTS_PL_BONUS	100
#define POINTS_DIE		20

/* Top-left coordinates of button palette */
#define MENU_TOP		(GP142_YMAX - 35)
#define MENU_LEFT		(-GP142_XMAX + 5)
#define MENU_FONTSIZE	12

#define MENU_TOP_PAUSED	(GP142_YMAX - 100)
#define MENU_LEFT_PAUSED	(-GP142_XMAX + 100)
#define MENU_FONTSIZE_PAUSED	12

/* Height and width of buttons in pixels */
#define BUTTON_HEIGHT  		20
#define BUTTON_WIDTH 		76
/* Height and width for pause screen buttons */
#define BUTTON_HEIGHT_PAUSED		20
#define BUTTON_WIDTH_PAUSED		76
/* Resume button from paused screen */
#define BUTTON_RESUME_WIDTH		152
#define BUTTON_RESUME_HEIGHT		40
#define BUTTON_RESUME_LEFT		110
#define BUTTON_RESUME_TOP		-240
#define BUTTON_RESUME_FONTSIZE	20
#define BUTTON_RESUME_OFFSET		10


#define BUTTON_CAR_X	MENU_LEFT + (BUTTON_WIDTH / 2)
#define BUTTON_CAR_Y	MENU_TOP
#define BUTTON_CAR_X2	(-MENU_LEFT) - (BUTTON_WIDTH / 2)
#define BUTTON_CAR_Y2	MENU_TOP

/* Buttons Constants */
#define NUM_BUTTONS 	15
#define BUTTON_STOP_GO		0
#define BUTTON_FASTER		1
#define BUTTON_SLOWER		2
#define BUTTON_REVERSE		3
#define BUTTON_COLOR		4
#define BUTTON_BIGGER		5
#define BUTTON_SMALLER		6
#define BUTTON_LANE_UP		7
#define BUTTON_LANE_DOWN		8
#define BUTTON_FIRE			9
#define BUTTON_SELECT_NEXT	10
#define BUTTON_SELECT_LAST	11
#define BUTTON_EXIST		12
#define BUTTON_RANDOMIZE		13
#define BUTTON_PAUSE		14

/* Paused Button Constants */
#define NUM_BUTTONS_PAUSED	8
#define BUTTON_RESUME_PAUSED		0	/* Special, keep at top */
#define BUTTON_MORE_CARS_PAUSED		1
#define BUTTON_LESS_CARS_PAUSED		2
#define BUTTON_MAX_CARS_PAUSED		3
#define BUTTON_MIN_CARS_PAUSED		4
#define BUTTON_RESET_SCORES_PAUSED		5
#define BUTTON_FAST_DRAW				6
#define BUTTON_SLOW_DRAW				7
/* !!! UPDATE HANDLE_BUTTON AND DRAW_BUTTONS WHEN ADDING BUTTONS !!! */



/* Player 1 Keyboard Constants */
#define KEY_STOP_GO		'z'
#define KEY_FASTER		'f'
#define KEY_SLOWER		's'
#define KEY_REVERSE		'c'
#define KEY_COLOR		'g'
#define KEY_BIGGER		(char) 22 /* insert key    */
#define KEY_SMALLER		(char) 7  /* delete key    */
#define KEY_LANE_UP		'e'
#define KEY_LANE_DOWN	'd'
#define KEY_FIRE		' '
#define KEY_SELECT_NEXT	'r'
#define KEY_SELECT_LAST	'w'
#define KEY_EXIST		'v'
#define KEY_RANDOMIZE	't'
#define KEY_PAUSE		'q'
/* Player 2 Keyboard Constants */
#define KEY2_STOP_GO	'0'
#define KEY2_FASTER		'6'
#define KEY2_SLOWER		'4'
#define KEY2_REVERSE	'2'
#define KEY2_COLOR		'/'
#define KEY2_BIGGER		(char) 18 /* page up key   */
#define KEY2_SMALLER	(char) 3  /* page down key */
#define KEY2_LANE_UP	'8'
#define KEY2_LANE_DOWN	'5'
#define KEY2_FIRE		(char) 13 /* enter key     */
#define KEY2_SELECT_NEXT	'9'
#define KEY2_SELECT_LAST	'7'
#define KEY2_EXIST		'-'
#define KEY2_RANDOMIZE	'+'
#define KEY2_PAUSE		'.'

/* Possible locations of mouse clicks: */
/*	Mouse click on a vehicle		*/
#define VEHICLE_CLICK 2
/*   Mouse click in some button	*/
#define BUTTON_CLICK 1
/*   Mouse click elsewhere		*/
#define OTHER_CLICK 0


/* Coordinates of objects in the drawing */

/* y-coordinate of the top of the ground in the landscape */
#define HORIZON (- 2 * (GP142_YMAX / 10))

/* Vertical center and width of highway */
#define HIGHWAY_CENTER	((-GP142_YMAX + HORIZON) / 3)
#define HIGHWAY_WIDTH	10
#define NUMBER_LANES	6

#define SUN_START_X		GP142_XMAX - 50
#define SUN_START_Y		GP142_YMAX - 35

#define EXPLOSION_FRAMES		10

#define MISSILE_FRAMES		30
#define MISSILE_SPEED		5
#define MISSILE_WIDTH		5
#define MISSILE_HEIGHT		3

/* the game_data struct, for easy passing of game settings to functions */
typedef struct {
	int paused;	/* pause game, halt most animation		*/
	int cars;		/* how many cars may be animated		*/
	int p1firing;	/* used to pass back and trigger firing	*/
	int p2firing;	/* used to pass back and trigger firing	*/
	int reset;	/* used to make sure cars exist		*/
	int score1;	/* player 1's score					*/
	int score2;	/* player 2's score					*/
	int fastdraw;	/*  T/F  ignore car draw order?		*/
} game_data;

/*****************************************************************************
     vehicle struct and vehicle/struct-related constants
*****************************************************************************/
typedef struct {
	char name;	/* not used much, name of vehicle		*/
	int type;		/* what kind of vehicle				*/
	int x;		/* screen x coordinate				*/
	int y;		/* screen y coordinate				*/
	int width;	/* vehicle width entire span			*/
	int height;	/* vehicle height entire span			*/
	int direction;	/* EAST or WEST movement				*/
	int speed;	/* magnitude of movement				*/
	int color;	/* car shell color					*/
	int moving;	/*  T/F  is vehicle moving?			*/
	int exists;	/*  T/F  is it in our environment?		*/
	int frame;	/* animation frame/explosion countdown	*/
	int player;	/* player controlling the vehicle, for score	*/
} vehicle;


/* vehicle player controller */
#define PLAYER_NONE	0
#define PLAYER_1	1
#define PLAYER_2	2

/* Vehicle type distinguishing  IE  car[0].type = VEHICLE_CAR  */
#define NUMBER_VEHICLE_TYPES	6
#define SPECIAL_VEHICLES		2
#define VEHICLE_CAR			0
#define VEHICLE_TRUCK		1
#define VEHICLE_PLANE		2
#define VEHICLE_CHOPPER		3
#define VEHICLE_EXPLOSION	4 /* SPECIAL - MUST REMAIN LAST */
#define VEHICLE_MISSILE		5 /* SPECIAL - MUST REMAIN LAST */

#define EAST	+1
#define WEST	-1

#define MAX_VEHICLES			59
#define FIRST_MISSILE_P1			51
#define LAST_MISSILE_P1			54
#define FIRST_MISSILE_P2			55
#define LAST_MISSILE_P2			58
#define VEHICLES_RESERVED		8	/* Reserved for Missiles     */
#define MAX_DECLARABLE_VEHICLES	MAX_VEHICLES - VEHICLES_RESERVED
#define MIN_DECLARABLE_VEHICLES	2	/* must have at least 2 cars */
#define STARTING_VEHICLES		12	/* how many to use at first  */

#define CAR_START_HEIGHT		22
#define CAR_START_WIDTH		38

#define TRUCK_START_HEIGHT	28
#define TRUCK_START_WIDTH	42

#define PLANE_START_HEIGHT	32
#define PLANE_START_WIDTH	54

#define CHOPPER_START_HEIGHT	24
#define CHOPPER_START_WIDTH	40

/* vehicle modification constraints */
#define LOW_GROUND_Y		HIGHWAY_CENTER - 8 * HIGHWAY_WIDTH
#define HIGH_GROUND_Y		HIGHWAY_CENTER + 2 * HIGHWAY_WIDTH
#define LOW_AIR_Y			HIGHWAY_CENTER - 8 * HIGHWAY_WIDTH
#define HIGH_AIR_Y			(GP142_YMAX / 10) * 10 - 10 /* make in slot */
#define MIN_VEHICLE_WIDTH	1  /* dont bother with max size  :) */
#define MIN_VEHICLE_HEIGHT	1  /* very large vehicles are fun   */
#define MAX_CAR_SPEED		20
#define MIN_CAR_SPEED		0
#define MAX_TRUCK_SPEED		20
#define MIN_TRUCK_SPEED		0
#define MAX_CHOPPER_SPEED	30
#define MIN_CHOPPER_SPEED	0
#define MAX_PLANE_SPEED		40
#define MIN_PLANE_SPEED		5

/* random creation constants */
#define CAR_MIN_SPEED		2
#define CAR_MAX_SPEED		17
#define TRUCK_MIN_SPEED		2
#define TRUCK_MAX_SPEED		17
#define PLANE_MIN_SPEED		9
#define PLANE_MAX_SPEED		23
#define CHOPPER_MIN_SPEED	5
#define CHOPPER_MAX_SPEED	19
#define RAND_CAR_RANGE		(CAR_MAX_SPEED		- CAR_MIN_SPEED	+ 1)
#define RAND_TRUCK_RANGE		(TRUCK_MAX_SPEED	- TRUCK_MIN_SPEED	+ 1)
#define RAND_PLANE_RANGE		(PLANE_MAX_SPEED	- PLANE_MIN_SPEED	+ 1)
#define RAND_CHOPPER_RANGE	(CHOPPER_MAX_SPEED	- CHOPPER_MIN_SPEED	+ 1)

/*****************************************************************************
     FUNCTION PROTOTYPES
*****************************************************************************/

/* Initialize random number generator.  [Ignore or		*/
/* remove if your program doesn't use random numbers.]	*/
void initialize_random_numbers();

/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y);

/* Draw the buttons */
void draw_buttons();
/* Draw the car selection above the buttons */
void draw_button_car(int type, int width, int height, int color, int dir, int player);

/* Return kind of mouse click (in a button, etc.) */
int  classify_mouse_click(int mouse_x, int mouse_y, vehicle car[]);

/* Process button click at location x, y.  Update car speed,  */
/* direction, color, and whether traffic is moving as needed. */
void handle_button(int x, int y, vehicle *car, int *car_selected, int *car_selected2, game_data *settings);
/* Process button click at location x, y, while game paused   */
void handle_button_paused(int x, int y, game_data *settings);

/* Update sun coordinates sun_x, sun_y for one tick */
void move_sun(int *sun_x, int *sun_y);

/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick. */
void move_traffic(int *x, int *y, int *direction, int speed);

/* Draw a single car of the given color with horizontal */
/* center x and bottom y,  etc for other vehicle types  */
void draw_car(		int x, int y, int width, int height, int color		);
void draw_truck(	int x, int y, int width, int height, int color, int dir);
void draw_plane(	int x, int y, int width, int height, int color, int dir);
void draw_chopper(	int x, int y, int width, int height, int color, int dir);
void draw_explosion(int x, int y, int width, int height				);
void draw_missile(	int x, int y, int width, int height, int dir			);

/* Display error message if internal error is detected. */
void something_is_wrong (void);

/* Replace a new vehicle with the given vehicle */
void randomize_vehicle(vehicle *car);

/* Display the 2 scores */
void draw_scores(int score1, int score2);

/* Pause game loop */
void pause_game(void);



/*****************************************************************************
     MAIN
*****************************************************************************/

/*
 * Main program contents:
 *   - Major program variables that persist throughout execution
 *   - Event loop that processes GP142 events until the user quits,
 *     and updates variables and redraws the window as needed.
 */
int main(void)
{
	int i, j;		/* temp vars	*/
	int force;	/* explosion composite force	*/
	/* Event handling and buttons: */
	int  quit;			/* = "user has selected quit"				*/
	int  event;			/* the most recent GP142 event			*/
	char key_pressed;		/* last keyboard character from key event	*/
	int  mouse_x, mouse_y;	/* x and y coordinates of latest mouse event	*/
	int  click_kind;		/* kind of mouse click (button, etc.)		*/
	int	car_selected  = 0;	/* Which car player 1 is controlling		*/
	int  car_selected2 = 1;	/* Which car player 2 is controlling		*/
	int  found;			/* Used for finding a missile shot			*/

	/* Information about the sun: */
	int sun_x, sun_y;		/* screen x and y coordinates */
	
	/* (Type game_data as defined earlier...)	*/
	game_data settings;
	/* (Type vehicle as defined earlier...)		*/
	vehicle car[MAX_VEHICLES];
	

	initialize_random_numbers();
	
	/* Initialize graphics package, clear the window,	*/
	/* turn logging off, and start animation.		*/
	GP142_open();
	GP142_logging(LOG_OFF);
	GP142_clear();
	GP142_animate(ANI_RUN);
	

	/* set initial location of the sun */
	sun_x = SUN_START_X;
	sun_y = SUN_START_Y;
	
	/* initial game settings */
	settings.paused = FALSE;
	settings.cars = STARTING_VEHICLES;
	settings.p1firing = FALSE;
	settings.p2firing = FALSE;
	settings.reset = FALSE;
	settings.score1 = 0;
	settings.score2 = 0;
	settings.fastdraw = FALSE;
	
	/* initialize all moving "vehicles", including reserved */
	for(i = 0; i < MAX_VEHICLES; i++) {
		car[i].exists = FALSE;
		car[i].moving = TRUE;
		car[i].frame = 0;
		car[i].player = PLAYER_NONE;
		car[i].direction = 1;
		car[i].color = 1;
		car[i].speed = 1;
		car[i].type = 1;
		car[i].x = 0;
		car[i].y = 0;
	}
	/* set initial conditions for vehicles */
	for(i = 0; i < settings.cars; i++) {
		/* let cars exist, but not special yet */
		car[i].exists = TRUE;
		randomize_vehicle(&car[i]);
	}
	/* initialize missile data */
	for(i = FIRST_MISSILE_P1; i <= LAST_MISSILE_P2; i++) {
		car[i].width = MISSILE_WIDTH;
		car[i].height = MISSILE_HEIGHT;
		car[i].type = VEHICLE_MISSILE;
	}
	

	/* <<< SPLASH SCREEN WILL GO HERE IF ADDED >>> */

	
	/*
	 * Main event loop:
	 * ---- ----- -----
	 * Wait for the next user action, decode it, and call an
	 * appropriate function to handle it.  Repeat until the
	 * user selects quit.
	 */
	quit = FALSE;
	while (!quit) {
		/* get next event */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_pressed);
		car[car_selected].player = PLAYER_1;
		car[car_selected2].player = PLAYER_2;
		
		/* perform appropriate action depending on kind of event */
		switch (event) {
		case GP142_QUIT:
			/* Quit selected */
			quit = TRUE;
			break;
			
		case GP142_KBD:
			/* Key pressed. Determine player 1 or 2 action. */
			switch (key_pressed) {
			case KEY_STOP_GO:
				if (car[car_selected].moving == TRUE) {
					car[car_selected].moving = FALSE;
				} else {
					car[car_selected].moving = TRUE;
				}
				break;
			case KEY_FASTER:
				car[car_selected].speed = car[car_selected].speed + 1;
				switch (car[car_selected].type) {
				case VEHICLE_CAR:
					if (car[car_selected].speed > MAX_CAR_SPEED)
						car[car_selected].speed = MAX_CAR_SPEED;
					break;
				case VEHICLE_TRUCK:
					if (car[car_selected].speed > MAX_TRUCK_SPEED)
						car[car_selected].speed = MAX_TRUCK_SPEED;
					break;
				case VEHICLE_CHOPPER:
					if (car[car_selected].speed > MAX_CHOPPER_SPEED)
						car[car_selected].speed = MAX_CHOPPER_SPEED;
					break;
				case VEHICLE_PLANE:
					if (car[car_selected].speed > MAX_PLANE_SPEED)
						car[car_selected].speed = MAX_PLANE_SPEED;
					break;
				default:
					break;
				}
				break;
			case KEY_SLOWER:
				car[car_selected].speed = car[car_selected].speed - 1;
				switch (car[car_selected].type) {
				case VEHICLE_CAR:
					if (car[car_selected].speed < MIN_CAR_SPEED)
						car[car_selected].speed = MIN_CAR_SPEED;
					break;
				case VEHICLE_TRUCK:
					if (car[car_selected].speed < MIN_TRUCK_SPEED)
						car[car_selected].speed = MIN_TRUCK_SPEED;
					break;
				case VEHICLE_CHOPPER:
					if (car[car_selected].speed < MIN_CHOPPER_SPEED)
						car[car_selected].speed = MIN_CHOPPER_SPEED;
					break;
				case VEHICLE_PLANE:
					if (car[car_selected].speed < MIN_PLANE_SPEED)
						car[car_selected].speed = MIN_PLANE_SPEED;
					break;
				default:
					break;
				}
				break;
			case KEY_REVERSE:
				car[car_selected].direction = - car[car_selected].direction;
				break;
			case KEY_COLOR:
				car[car_selected].color = (car[car_selected].color+1) % MAX_COLORS;
				break;
			case KEY_BIGGER:
				car[car_selected].width = car[car_selected].width + 2;
				car[car_selected].height = car[car_selected].height + 1;
				break;
			case KEY_SMALLER:
				car[car_selected].width = car[car_selected].width - 2;
				if (car[car_selected].width < MIN_VEHICLE_WIDTH) 
					car[car_selected].width = MIN_VEHICLE_WIDTH;
				car[car_selected].height = car[car_selected].height - 1;
				if (car[car_selected].height < MIN_VEHICLE_WIDTH) 
					car[car_selected].height = MIN_VEHICLE_WIDTH;
				break;
			case KEY_LANE_UP:
				car[car_selected].y = car[car_selected].y + HIGHWAY_WIDTH;
				if(		(car[car_selected].type == VEHICLE_CAR || 
						 car[car_selected].type == VEHICLE_TRUCK)  &&
						(car[car_selected].y > HIGH_GROUND_Y)        )
					car[car_selected].y = HIGH_GROUND_Y;
				if(		(car[car_selected].type == VEHICLE_PLANE || 
						 car[car_selected].type == VEHICLE_CHOPPER)  &&
						(car[car_selected].y > HIGH_AIR_Y)        )
					car[car_selected].y = HIGH_AIR_Y;
				break;
			case KEY_LANE_DOWN:
				car[car_selected].y = car[car_selected].y - HIGHWAY_WIDTH;
				if(		(car[car_selected].type == VEHICLE_CAR || 
						 car[car_selected].type == VEHICLE_TRUCK)  &&
						(car[car_selected].y < LOW_GROUND_Y)        )
					car[car_selected].y = LOW_GROUND_Y;
				if(		(car[car_selected].type == VEHICLE_PLANE || 
						 car[car_selected].type == VEHICLE_CHOPPER)  &&
						(car[car_selected].y < LOW_AIR_Y)        )
					car[car_selected].y = LOW_AIR_Y;
				break;
			case KEY_FIRE:
				settings.p1firing = TRUE;
				break;
			case KEY_SELECT_NEXT:
				car[car_selected].player = PLAYER_NONE;
				car_selected += 1;
				if (car[car_selected].player == PLAYER_2)
					car_selected += 1;
				if (car_selected > (settings.cars - 1))
					car_selected = 0;
				car[car_selected].player = PLAYER_1;
				break;
			case KEY_SELECT_LAST:
				car[car_selected].player = PLAYER_NONE;
				car_selected -= 1;
				if (car[car_selected].player == PLAYER_2)
					car_selected -= 1;
				if (car_selected < 0)
					car_selected = (settings.cars - 1);
				car[car_selected].player = PLAYER_1;
				break;
			case KEY_EXIST:
				if (car[car_selected].exists == TRUE) {
					car[car_selected].exists = FALSE;
				} else {
					car[car_selected].exists = TRUE;
				}
				break;
			case KEY_RANDOMIZE:
				randomize_vehicle(&car[car_selected]);
				break;
			case KEY_PAUSE:
				settings.paused = TRUE;
				pause_game();
				settings.reset = TRUE;
				break;
			case KEY2_STOP_GO:
				if (car[car_selected2].moving == TRUE) {
					car[car_selected2].moving = FALSE;
				} else {
					car[car_selected2].moving = TRUE;
				}
				break;
			case KEY2_FASTER:
				car[car_selected2].speed = car[car_selected2].speed + 1;
				switch (car[car_selected2].type) {
				case VEHICLE_CAR:
					if (car[car_selected2].speed > MAX_CAR_SPEED)
						car[car_selected2].speed = MAX_CAR_SPEED;
					break;
				case VEHICLE_TRUCK:
					if (car[car_selected2].speed > MAX_TRUCK_SPEED)
						car[car_selected2].speed = MAX_TRUCK_SPEED;
					break;
				case VEHICLE_CHOPPER:
					if (car[car_selected2].speed > MAX_CHOPPER_SPEED)
						car[car_selected2].speed = MAX_CHOPPER_SPEED;
					break;
				case VEHICLE_PLANE:
					if (car[car_selected2].speed > MAX_PLANE_SPEED)
						car[car_selected2].speed = MAX_PLANE_SPEED;
					break;
				default:
					break;
				}
				break;
			case KEY2_SLOWER:
				car[car_selected2].speed = car[car_selected2].speed - 1;
				switch (car[car_selected2].type) {
				case VEHICLE_CAR:
					if (car[car_selected2].speed < MIN_CAR_SPEED)
						car[car_selected2].speed = MIN_CAR_SPEED;
					break;
				case VEHICLE_TRUCK:
					if (car[car_selected2].speed < MIN_TRUCK_SPEED)
						car[car_selected2].speed = MIN_TRUCK_SPEED;
					break;
				case VEHICLE_CHOPPER:
					if (car[car_selected2].speed < MIN_CHOPPER_SPEED)
						car[car_selected2].speed = MIN_CHOPPER_SPEED;
					break;
				case VEHICLE_PLANE:
					if (car[car_selected2].speed < MIN_PLANE_SPEED)
						car[car_selected2].speed = MIN_PLANE_SPEED;
					break;
				default:
					break;
				}
				break;
			case KEY2_REVERSE:
				car[car_selected2].direction = - car[car_selected2].direction;
				break;
			case KEY2_COLOR:
				car[car_selected2].color = (car[car_selected2].color+1) % MAX_COLORS;
				break;
			case KEY2_BIGGER:
				car[car_selected2].width = car[car_selected2].width + 2;
				car[car_selected2].height = car[car_selected2].height + 1;
				break;
			case KEY2_SMALLER:
				car[car_selected2].width = car[car_selected2].width - 2;
				if (car[car_selected2].width < MIN_VEHICLE_WIDTH) 
					car[car_selected2].width = MIN_VEHICLE_WIDTH;
				car[car_selected2].height = car[car_selected2].height - 1;
				if (car[car_selected2].height < MIN_VEHICLE_HEIGHT) 
					car[car_selected2].height = MIN_VEHICLE_HEIGHT;
				break;
			case KEY2_LANE_UP:
				car[car_selected2].y = car[car_selected2].y + HIGHWAY_WIDTH;
				if(		(car[car_selected2].type == VEHICLE_CAR || 
						 car[car_selected2].type == VEHICLE_TRUCK)  &&
						(car[car_selected2].y > HIGH_GROUND_Y)        )
					car[car_selected2].y = HIGH_GROUND_Y;
				if(		(car[car_selected2].type == VEHICLE_PLANE || 
						 car[car_selected2].type == VEHICLE_CHOPPER)  &&
						(car[car_selected2].y > HIGH_AIR_Y)        )
					car[car_selected2].y = HIGH_AIR_Y;
				break;
			case KEY2_LANE_DOWN:
				car[car_selected2].y = car[car_selected2].y - HIGHWAY_WIDTH;
				if(		(car[car_selected2].type == VEHICLE_CAR || 
						 car[car_selected2].type == VEHICLE_TRUCK)  &&
						(car[car_selected2].y < LOW_GROUND_Y)        )
					car[car_selected2].y = LOW_GROUND_Y;
				if(		(car[car_selected2].type == VEHICLE_PLANE || 
						 car[car_selected2].type == VEHICLE_CHOPPER)  &&
						(car[car_selected2].y < LOW_AIR_Y)        )
					car[car_selected2].y = LOW_AIR_Y;
				break;
			case KEY2_FIRE:
				settings.p2firing = TRUE;
				break;
			case KEY2_SELECT_NEXT:
				car[car_selected2].player = PLAYER_NONE;
				car_selected2 += 1;
				if (car[car_selected2].player == PLAYER_1)
					car_selected2 += 1;
				if (car_selected2 > (settings.cars - 1))
					car_selected2 = 0;
				car[car_selected2].player = PLAYER_2;
				break;
			case KEY2_SELECT_LAST:
				car[car_selected2].player = PLAYER_NONE;
				car_selected2 -= 1;
				if (car[car_selected2].player == PLAYER_1)
					car_selected2 -= 1;
				if (car_selected2 < 0)
					car_selected2 = (settings.cars - 1);
				car[car_selected2].player = PLAYER_2;
				break;
			case KEY2_EXIST:
				if (car[car_selected2].exists == TRUE) {
					car[car_selected2].exists = FALSE;
				} else {
					car[car_selected2].exists = TRUE;
				}
				break;
			case KEY2_RANDOMIZE:
				randomize_vehicle(&car[car_selected2]);
				break;
			case KEY2_PAUSE:
				settings.paused = TRUE;
				pause_game();
				settings.reset = TRUE;
				break;
			default:
				break;
			}
			break;
			
		case GP142_MOUSE:
			/* Mouse click.  Determine if in a menu button	*/
			/* or elsewhere and process accordingly.		*/
			if (settings.paused == FALSE) {
				click_kind = classify_mouse_click(mouse_x, mouse_y, car);
				if (click_kind == BUTTON_CLICK) {
					handle_button(mouse_x, mouse_y, &car[car_selected], 
								&car_selected, &car_selected2, &settings);
				} else {
					if (click_kind >= VEHICLE_CLICK)
						/* Returned was V_CLICK + index of var clicked */
						car_selected = click_kind - VEHICLE_CLICK;
				}
			} else {   /* if settings.paused == TRUE: */
				pause_game();
				handle_button_paused(mouse_x, mouse_y, &settings);
			}
			break;
			
		case GP142_PERIODIC:
			/* Timer interval event.  Update everything	*/
			/* that needs to change as time advances.	*/
			
			/* let sun move while paused, for fun   :)	*/
			move_sun(&sun_x, &sun_y);
			
			if (settings.paused == FALSE) {
				
				/* make sure only vehicles actually selected by players */
				/*    are marked with .player */
				for(i = 0; i < MAX_DECLARABLE_VEHICLES; i++) {
					car[i].player = PLAYER_NONE;
				}
				car[car_selected].player = PLAYER_1;
				car[car_selected2].player = PLAYER_2;

				for(i = 0; i < MAX_VEHICLES ; i++) {
					if (car[i].moving) {
						move_traffic(&car[i].x, &car[i].y, 
								&car[i].direction, car[i].speed);
					}
					/* For each car, see if there are any collisions with any 
					   other cars.  If there are, create an explosion out     
					   of 1 of them and eliminate other				   */
					if (car[i].exists) {
						for(j = i + 1; j < MAX_VEHICLES; j++) {
							if (car[j].exists == TRUE) {
								if	(	(car[i].x + (car[i].width / 2))  > 
										(car[j].x - (car[j].width / 2))  &&
										(car[i].x - (car[i].width / 2))  <
										(car[j].x + (car[j].width / 2))  &&
										(car[i].y == car[j].y) ) {
									/* Calculate point gains, if any */
									/* gain points for shooting vehicles */
									if (car[j].type == VEHICLE_MISSILE) {
										if (car[i].type != VEHICLE_MISSILE  && 
												car[i].type != VEHICLE_EXPLOSION) {
											if (car[j].player == PLAYER_1)
 												settings.score1 += POINTS_CAR;
											if (car[j].player == PLAYER_2)
												settings.score2 += POINTS_CAR;
											if (car[i].player == PLAYER_2)
												settings.score1 += POINTS_PL_BONUS;
											if (car[i].player == PLAYER_1)
												settings.score2 += POINTS_PL_BONUS;
										}
									}
									/* lose points if you blow up in all collisions */
									if (	(car[i].player == PLAYER_1) && 
											(car[i].type != VEHICLE_EXPLOSION) &&
											(car[i].type != VEHICLE_MISSILE)  )
										settings.score1 -= POINTS_DIE;
									if (	(car[j].player == PLAYER_1) && 
											(car[j].type != VEHICLE_EXPLOSION) &&
											(car[j].type != VEHICLE_MISSILE)  )
										settings.score1 -= POINTS_DIE;
									if (	(car[i].player == PLAYER_2) && 
											(car[i].type != VEHICLE_EXPLOSION) &&
											(car[i].type != VEHICLE_MISSILE)  )
										settings.score2 -= POINTS_DIE;
									if (	(car[j].player == PLAYER_2) && 
											(car[j].type != VEHICLE_EXPLOSION) &&
											(car[j].type != VEHICLE_MISSILE)  )
										settings.score2 -= POINTS_DIE;

									car[i].type = VEHICLE_EXPLOSION;
									car[j].exists = FALSE;
									/* determine composite exploded characteristics */
									/* If car is Stopped, treat as if has no speed */
									car[i].x = (car[i].x + car[j].x) / 2;
									if (car[i].moving == FALSE)
										car[i].speed = 0;
									if (car[j].moving == FALSE)
										car[j].speed = 0;
									force  =	car[i].speed * car[i].direction * 
											car[i].width * car[i].height +
											car[j].speed * car[j].direction *
											car[j].width * car[j].height;
									/* Positive force direction is East */
									if (force >= 0)  
										car[i].direction = EAST;
									else
										car[i].direction = WEST;
									/* Explosion keeps 2 of larger sizes */
									if (car[j].width > car[i].width)
										car[i].width  = car[j].width;
									if (car[j].height > car[i].height)
										car[i].height = car[j].height;
									car[i].speed = abs((force / (car[i].width * car[i].height))) / 2;
									if (!(car[j].type == VEHICLE_MISSILE))
										randomize_vehicle(&car[j]);
								} /* endif cars collided */
							} /* endif car[j] exists */
						}
					} /* endif car[i] exists */
				} 
			} /* end if not paused */
			break;
			
		default:
			/* unknown event */
			break;
			
		}  /*end switch */
		
		
		if (settings.p1firing == TRUE && car[car_selected].exists) {
			/* find an unused p1 missile slot and create missile */
			i = FIRST_MISSILE_P1; 
			found = FALSE;
			while(!found) {
				if (car[i].exists == FALSE) {
					settings.p1firing = FALSE;
					found = TRUE;
					car[i].y = car[car_selected].y;
					/* Place shot just far enough not to run over */
					car[i].x = car[car_selected].x + 
							car[car_selected].direction * 
							car[car_selected].width +  
							car[car_selected].direction * 
							(car[car_selected].speed + 1);
					car[i].exists = TRUE;
					car[i].direction = car[car_selected].direction;
					car[i].speed = car[car_selected].speed + MISSILE_SPEED;
					/* If player is sitting still, missile is slower */
					if (!car[car_selected].moving)
						car[i].speed = MISSILE_SPEED;
					car[i].player = PLAYER_1;
					car[i].frame = 1;
				} else {
					if (i == LAST_MISSILE_P1)
						found = TRUE; /* "pretend found" and not fire: */
					i++;
				}
			}
		}
		if (settings.p2firing == TRUE && car[car_selected2].exists ) {
			/* find an unused p2 missile slot and create missile */
			i = FIRST_MISSILE_P2; 
			found = FALSE;
			while(!found) {
				if (car[i].exists == FALSE) {
					settings.p2firing = FALSE;
					found = TRUE;
					car[i].y = car[car_selected2].y;
					car[i].x = car[car_selected2].x + 
							car[car_selected2].direction * 
							car[car_selected2].width +  
							car[car_selected2].direction * 
							(car[car_selected2].speed + 1);
					car[i].exists = TRUE;
					car[i].direction = car[car_selected2].direction;
					car[i].speed = car[car_selected2].speed + MISSILE_SPEED;
					/* If player is sitting still, missile is slower */
					if (!car[car_selected2].moving)
						car[i].speed = MISSILE_SPEED;
					car[i].player = PLAYER_2;
				} else {
					if (i == LAST_MISSILE_P2)
						/* "pretend found" and not fire: */
						found = TRUE;
					i++;
				}
			}
		}
		
		if (settings.reset) {
			settings.reset = FALSE;
			
			car_selected = 0;
			car_selected2 = 1;
			car[0].player = PLAYER_1;
			car[1].player = PLAYER_2;

			for(i = 0; i < MAX_DECLARABLE_VEHICLES; i++) {
				if (i < settings.cars) {
					car[i].exists = TRUE;
					car[i].moving = TRUE;
					randomize_vehicle(&car[i]);
				} else {
					car[i].exists = FALSE;
				}
			}
		}
		
		/* Redraw everything */
		draw_landscape(sun_x, sun_y);
		draw_buttons( );
		draw_button_car( car[car_selected].type, 
				car[car_selected].width, car[car_selected].height, 
				car[car_selected].color, car[car_selected].direction, 
				PLAYER_1);
		draw_button_car( car[car_selected2].type, 
				car[car_selected2].width, car[car_selected2].height, 
				car[car_selected2].color, car[car_selected2].direction,
				PLAYER_2);
		if (settings.paused == FALSE) {
			if (!settings.fastdraw) {
				/* progress from top to bottom, drawing vehicles */
				/* in this order, the "closer" will draw on top  */
				for(j = HIGH_AIR_Y; j >= LOW_GROUND_Y; j -= HIGHWAY_WIDTH) {
					for(i = 0; i < settings.cars; i++) {
						if (car[i].exists && car[i].y == j) {
							if (car[i].type == VEHICLE_CAR)
								draw_car( car[i].x, car[i].y, car[i].width, car[i].height,
										car[i].color);
							else if (car[i].type == VEHICLE_TRUCK)
								draw_truck( car[i].x, car[i].y, car[i].width, car[i].height,
										car[i].color, car[i].direction );
							else if (car[i].type == VEHICLE_PLANE)
								draw_plane( car[i].x, car[i].y, car[i].width, car[i].height,
										car[i].color, car[i].direction );
							else if (car[i].type == VEHICLE_CHOPPER)
								draw_chopper( car[i].x, car[i].y, car[i].width, car[i].height,
										car[i].color, car[i].direction );
							else if (car[i].type == VEHICLE_EXPLOSION) {
								draw_explosion( car[i].x, car[i].y, car[i].width, car[i].height);
								car[i].frame += 1;
								if (car[i].frame > EXPLOSION_FRAMES)
									if (!(car[i].type == VEHICLE_MISSILE))
										randomize_vehicle(&car[i]);
							} 
						} /* end if car exists */
					} /* end for each car */
				} /* end for each y position */
			} else {
				/* Draw each vehicle, straight through, without regard to */
				/*   which will appear 'closer' than the other	 */
				for(i = 0; i < settings.cars; i++) {
					if (car[i].exists) {
						if (car[i].type == VEHICLE_CAR)
							draw_car( car[i].x, car[i].y, car[i].width, car[i].height,
									car[i].color);
						else if (car[i].type == VEHICLE_TRUCK)
							draw_truck( car[i].x, car[i].y, car[i].width, car[i].height,
									car[i].color, car[i].direction );
						else if (car[i].type == VEHICLE_PLANE)
							draw_plane( car[i].x, car[i].y, car[i].width, car[i].height,
									car[i].color, car[i].direction );
						else if (car[i].type == VEHICLE_CHOPPER)
							draw_chopper( car[i].x, car[i].y, car[i].width, car[i].height,
									car[i].color, car[i].direction );
						else if (car[i].type == VEHICLE_EXPLOSION) {
							draw_explosion( car[i].x, car[i].y, car[i].width, car[i].height);
							car[i].frame += 1;
							if (car[i].frame > EXPLOSION_FRAMES)
								if (!(car[i].type == VEHICLE_MISSILE))
									randomize_vehicle(&car[i]);
						} 
					} /* end if car exists */
				} /* end for each car */
			}
			for(i = FIRST_MISSILE_P1;i <= LAST_MISSILE_P2;i++) {
				if (car[i].exists == TRUE) {
					draw_missile( car[i].x, car[i].y, car[i].width, car[i].height,
							car[i].direction);
					car[i].frame += 1;
					if (car[i].frame > MISSILE_FRAMES) {
						car[i].exists = FALSE;
						car[i].frame = 0;
					}
				} /* end if missile exists */
			}
			if (settings.score1 < 0)
				settings.score1 = 0;
			if (settings.score2 < 0)
				settings.score2 = 0;
			draw_scores(settings.score1, settings.score2);
		} else {  /* settings.paused == TRUE: */
			pause_game();
		}
	}  /* end Main event loop */
	
	
	/* Termination: shut down graphics window and exit */
	GP142_close();
	return 0;
}  /*end main */





/*****************************************************************************
     BACKGROUND LANDSCAPE
*****************************************************************************/


/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y)
{
	int text_offset = -GP142_XMAX + 15;	/* horizontal offset of text */
	int text_size   = 12;				/* size of text font */

	/* Draw the sky */
	GP142_rectangleXY(ICE_BLUE, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, GP142_YMAX, FILL);
	
	/* Draw the sun */
	GP142_circleXY(YELLOW, sun_x, sun_y , 20);
	
	/* Draw the ground */
	GP142_rectangleXY(BROWN, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, HORIZON, FILL);
	/* road 1 */
	GP142_rectangleXY(WHITE, 
				-GP142_XMAX, HIGHWAY_CENTER - 2 * HIGHWAY_WIDTH,
				 GP142_XMAX, HIGHWAY_CENTER + HIGHWAY_WIDTH,
				 FILL);
	/* road divider 1 */
	GP142_rectangleXY(BLACK, 
				-GP142_XMAX, HIGHWAY_CENTER, 
				 GP142_XMAX, HIGHWAY_CENTER, 
				 FILL);
	/* road divider 2 */
	GP142_rectangleXY(BLACK, 
				-GP142_XMAX, HIGHWAY_CENTER - HIGHWAY_WIDTH, 
				 GP142_XMAX, HIGHWAY_CENTER - HIGHWAY_WIDTH,
				 FILL);
	/* road 2 */
	GP142_rectangleXY(WHITE, 
				-GP142_XMAX, HIGHWAY_CENTER - 6 * HIGHWAY_WIDTH,
				 GP142_XMAX, HIGHWAY_CENTER - 3 * HIGHWAY_WIDTH,
				 FILL);
	/* road divider 3 */
	GP142_rectangleXY(BLACK, 
				-GP142_XMAX, HIGHWAY_CENTER - 4 * HIGHWAY_WIDTH, 
				 GP142_XMAX, HIGHWAY_CENTER - 4 * HIGHWAY_WIDTH, 
				 FILL);
	/* road divider 4 */
	GP142_rectangleXY(BLACK, 
				-GP142_XMAX, HIGHWAY_CENTER - 5 * HIGHWAY_WIDTH, 
				 GP142_XMAX, HIGHWAY_CENTER - 5 * HIGHWAY_WIDTH, 
				 FILL);
	
	GP142_printfXY (BLACK, -120, -GP142_YMAX + 30, text_size,
		"Click a button to control the car.\nPAUSE for player 1 and 2 keyboard controls.");
	
	
	/*Draw some mountains */
	GP142_triangleXY(MED_GRAY,
		-GP142_XMAX, HORIZON,
		GP142_XMAX, HORIZON,
		0, (4 * GP142_YMAX)/5 ,
		FILL);
	
	GP142_triangleXY(OLIVE,
		-( 5 * GP142_XMAX/4), HORIZON,
		(GP142_XMAX)/3  , HORIZON,
		-(3* GP142_XMAX)/4, (2 * GP142_YMAX)/3 ,
		FILL);
	
	GP142_triangleXY(FOREST_GREEN,
		-GP142_XMAX / 3 , HORIZON,
		GP142_XMAX, HORIZON,
		GP142_XMAX, (2 * GP142_YMAX)/3 ,
		FILL);

} /*end draw_landscape */



/*****************************************************************************
     BUTTONS AND MOUSE CLICKS
*****************************************************************************/


/* Draws the buttons */
void draw_buttons( )
{
	int i;
	int text_offset = MENU_LEFT+10;	/* offset of button text in button */
	
	/* draw rectangles for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		GP142_rectangleXY(BLACK,
			MENU_LEFT,
			MENU_TOP - i * BUTTON_HEIGHT,
			MENU_LEFT + BUTTON_WIDTH,
			MENU_TOP - (i + 1)*BUTTON_HEIGHT, 1);
	}
	
	/* display button text */
	i = MENU_TOP - (BUTTON_HEIGHT/2) - (MENU_FONTSIZE/2);
	GP142_printfXY(FOREST_GREEN, text_offset, i, MENU_FONTSIZE, "Go");	
	GP142_printfXY(BLACK, text_offset + 20, i, MENU_FONTSIZE, "/");
	GP142_printfXY(RED, text_offset + 25, i, MENU_FONTSIZE, "Stop");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE, "  Faster");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE, "  Slower");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE, "Reverse");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE, "New Color");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE+1, "  Enlarge");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE-1, "  Shrink");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE, "Lane Up");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE, "Lane Down");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLUE,   text_offset, i, MENU_FONTSIZE, "  FIRE!");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE, "Select Next");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE, "Select Last");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE, "Toggle Exist");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE, "Randomize");
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE, "  PAUSE");
	
}

/* Draws the buttons */
void draw_buttons_paused( )
{
	int i;
	/* offset of button text in button */
	int text_offset = MENU_LEFT_PAUSED + 10;	
	
	/* draw some rectangles so we can read stuff */
	GP142_rectangleXY(GOLD, -115, -250, 290, 150,   FILL);
	GP142_rectangleXY(OLIVE, -GP142_XMAX, -GP142_YMAX, 
			GP142_XMAX, (60 - GP142_YMAX),    FILL);

	/* draw rectangles for each button */
	for(i = 0; i < (NUM_BUTTONS_PAUSED - 1); i++) {
		/* background to make this stand out too */
		GP142_rectangleXY(WHITE,
			MENU_LEFT_PAUSED,
			MENU_TOP_PAUSED - i * BUTTON_HEIGHT_PAUSED,
			MENU_LEFT_PAUSED + BUTTON_WIDTH_PAUSED,
			MENU_TOP_PAUSED - (i + 1) * BUTTON_HEIGHT_PAUSED, FILL);
		GP142_rectangleXY(BLACK,
			MENU_LEFT_PAUSED,
			MENU_TOP_PAUSED - i * BUTTON_HEIGHT_PAUSED,
			MENU_LEFT_PAUSED + BUTTON_WIDTH_PAUSED,
			MENU_TOP_PAUSED - (i + 1) * BUTTON_HEIGHT_PAUSED, 1);
	}
	/* draw resume's rectangle */
	GP142_rectangleXY(BLACK,
			BUTTON_RESUME_LEFT,   BUTTON_RESUME_TOP, 
			BUTTON_RESUME_LEFT + BUTTON_RESUME_WIDTH,
			BUTTON_RESUME_TOP + BUTTON_RESUME_HEIGHT,  2);


	/* display button text */
	i = MENU_TOP_PAUSED - (BUTTON_HEIGHT_PAUSED/2) - (MENU_FONTSIZE_PAUSED/2);
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE_PAUSED, "More Cars");
	
	i -= BUTTON_HEIGHT_PAUSED;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE_PAUSED, "Less Cars");
	
	i -= BUTTON_HEIGHT_PAUSED;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE_PAUSED, "Max Cars");
	
	i -= BUTTON_HEIGHT_PAUSED;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE_PAUSED, "Min Cars");
	
	i -= BUTTON_HEIGHT_PAUSED;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE_PAUSED, "Reset Scores");
	
	i -= BUTTON_HEIGHT_PAUSED;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE_PAUSED, "Fast Draw");
	
	i -= BUTTON_HEIGHT_PAUSED;
	GP142_printfXY(BLACK, text_offset, i, MENU_FONTSIZE_PAUSED, "Slow Draw");
	
	GP142_printfXY(BLUE, (BUTTON_RESUME_LEFT + BUTTON_RESUME_OFFSET), 
			(BUTTON_RESUME_TOP + BUTTON_RESUME_OFFSET), 
			BUTTON_RESUME_FONTSIZE,					"  RESUME");
	
	GP142_printfXY(BLACK, -240, -220, MENU_FONTSIZE_PAUSED, 
		"*  Fast Draw = Speeds up drawing process by ignoring car draw order");
	GP142_printfXY(BLACK, -240, -240, MENU_FONTSIZE_PAUSED, 
		"   Slow Draw = Draw 'closer' vehicles to appear closer (default).");

	GP142_printfXY(BLUE, 0, 130, MENU_FONTSIZE_PAUSED + 6, "PLAYER 1 CONTROLS:");
	GP142_printfXY(BLACK, -110, 100, MENU_FONTSIZE_PAUSED, "   < Q >\n Pause   ");
	GP142_printfXY(BLACK,  -30, 100, MENU_FONTSIZE_PAUSED, "   < W >\nSelect Prev.");
	GP142_printfXY(BLACK,   50, 100, MENU_FONTSIZE_PAUSED, "   < E >\n Lane Up ");
	GP142_printfXY(BLACK,  130, 100, MENU_FONTSIZE_PAUSED, "   < R >\nSelect Next");
	GP142_printfXY(BLACK,  210, 100, MENU_FONTSIZE_PAUSED, "   < T >\nRandomize");
	GP142_printfXY(BLACK,  -80,  70, MENU_FONTSIZE_PAUSED, "   < A > ");
	GP142_printfXY(BLACK,    0,  70, MENU_FONTSIZE_PAUSED, "   < S >\n Slower  ");
	GP142_printfXY(BLACK,   80,  70, MENU_FONTSIZE_PAUSED, "   < D >\nLane Down");
	GP142_printfXY(BLACK,  160,  70, MENU_FONTSIZE_PAUSED, "   < F >\n Faster  ");
	GP142_printfXY(BLACK,  240,  70, MENU_FONTSIZE_PAUSED, "   < G >\n Color   ");
	GP142_printfXY(BLACK,  -50,  40, MENU_FONTSIZE_PAUSED, "   < Z >\nStop / Go");
	GP142_printfXY(BLACK,   30,  40, MENU_FONTSIZE_PAUSED, "   < X > ");
	GP142_printfXY(BLACK,  110,  40, MENU_FONTSIZE_PAUSED, "   < C >\n Reverse ");
	GP142_printfXY(BLACK,  190,  40, MENU_FONTSIZE_PAUSED, "   < V >\n Exist   ");
	GP142_printfXY(BLACK,  100,  10, MENU_FONTSIZE_PAUSED, "  <Space>: Fire  ");
	
	GP142_printfXY(BLUE, 0, -20, MENU_FONTSIZE_PAUSED + 6, "PLAYER 2 CONTROLS:");
	GP142_printfXY(BLACK,   40,  -40, MENU_FONTSIZE_PAUSED, "   < / >\n Color   ");
	GP142_printfXY(BLACK,  120,  -40, MENU_FONTSIZE_PAUSED, "   < * > ");
	GP142_printfXY(BLACK,  200,  -40, MENU_FONTSIZE_PAUSED, "   < - >\n Exist   ");
	GP142_printfXY(BLACK,  -40,  -70, MENU_FONTSIZE_PAUSED, "   < 7 >\nSelect Prev.");
	GP142_printfXY(BLACK,   40,  -70, MENU_FONTSIZE_PAUSED, "   < 8 >\n Lane Up ");
	GP142_printfXY(BLACK,  120,  -70, MENU_FONTSIZE_PAUSED, "   < 9 >\nSelect Next");
	GP142_printfXY(BLACK,  200,  -70, MENU_FONTSIZE_PAUSED, "   < + >.\nRandomize");
	GP142_printfXY(BLACK,  -40, -100, MENU_FONTSIZE_PAUSED, "   < 4 >\n Slower  ");
	GP142_printfXY(BLACK,   40, -100, MENU_FONTSIZE_PAUSED, "   < 5 >\nLane Down");
	GP142_printfXY(BLACK,  120, -100, MENU_FONTSIZE_PAUSED, "   < 6 >\n Faster  ");
	GP142_printfXY(BLACK,  -40, -140, MENU_FONTSIZE_PAUSED, "   < 1 >\n ");
	GP142_printfXY(BLACK,   40, -140, MENU_FONTSIZE_PAUSED, "   < 2 >\n Reverse ");
	GP142_printfXY(BLACK,  120, -140, MENU_FONTSIZE_PAUSED, "   < 3 >\n ");
	GP142_printfXY(BLACK,  -40, -170, MENU_FONTSIZE_PAUSED, "   < 0 >\nStop / Go");
	GP142_printfXY(BLACK,  120, -170, MENU_FONTSIZE_PAUSED, "   < . >\n Pause   ");
	GP142_printfXY(BLACK,  200, -170, MENU_FONTSIZE_PAUSED, "  <Enter>\n  Fire");
	
}

/* Classify mouse click at (mouse_x,mouse_y) and return kind */
int classify_mouse_click(int mouse_x, int mouse_y, vehicle car[])
{
	int i = 0;

	if ((mouse_x > MENU_LEFT) &&
			(mouse_x < MENU_LEFT + BUTTON_WIDTH) &&
			(mouse_y < MENU_TOP) &&
			(mouse_y > (MENU_TOP - NUM_BUTTONS*BUTTON_HEIGHT)))
		return BUTTON_CLICK;
	for(i = 0; i < MAX_DECLARABLE_VEHICLES; i++) {
		if (car[i].exists == TRUE) {
			if (		(mouse_x < (car[i].x + car[i].width)  ) &&
					(mouse_x > (car[i].x - car[i].width)  ) &&
					(mouse_y < (car[i].y + car[i].height) ) &&
					(mouse_y > (car[i].y ) ) ) {
				return (VEHICLE_CLICK + i);
			}
		} /* end if car exists */
	}
	return OTHER_CLICK;
}



/* Process button click at location x, y.  Update car speed,	*/
/* direction, color, and whether traffic is moving as needed.	*/
void handle_button(int x, int y, vehicle *car, int *car_selected, 
				int *car_selected2, game_data *settings) {
	int buttonNum;		/* number of selected button */

	/* Complain if x coordinate couldn't be inside a button */
	if (x < MENU_LEFT || x > MENU_LEFT + BUTTON_WIDTH)  {
		something_is_wrong();
		return;
	}
	
	/* Convert y coordinate to button number from top. */
	buttonNum = (MENU_TOP - y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum) {
	case BUTTON_STOP_GO:
		if ((*car).moving == TRUE) {
			(*car).moving = FALSE;
		} else {
			(*car).moving = TRUE;
		}
		break;
	case BUTTON_FASTER:
		/* speed up */
		(*car).speed = (*car).speed + 1;
		switch ((*car).type) {
		case VEHICLE_CAR:
			if ((*car).speed > MAX_CAR_SPEED)
				(*car).speed = MAX_CAR_SPEED;
			break;
		case VEHICLE_TRUCK:
			if ((*car).speed > MAX_TRUCK_SPEED)
				(*car).speed = MAX_TRUCK_SPEED;
			break;
		case VEHICLE_CHOPPER:
			if ((*car).speed > MAX_CHOPPER_SPEED)
				(*car).speed = MAX_CHOPPER_SPEED;
			break;
		case VEHICLE_PLANE:
			if ((*car).speed > MAX_PLANE_SPEED)
				(*car).speed = MAX_PLANE_SPEED;
			break;
		default:
			break;
		}
		break;
	case BUTTON_SLOWER:
		/* slow down */
		(*car).speed = (*car).speed - 1;
		switch ((*car).type) {
		case VEHICLE_CAR:
			if ((*car).speed < MIN_CAR_SPEED)
				(*car).speed = MIN_CAR_SPEED;
			break;
		case VEHICLE_TRUCK:
			if ((*car).speed < MIN_TRUCK_SPEED)
				(*car).speed = MIN_TRUCK_SPEED;
			break;
		case VEHICLE_CHOPPER:
			if ((*car).speed < MIN_CHOPPER_SPEED)
				(*car).speed = MIN_CHOPPER_SPEED;
			break;
		case VEHICLE_PLANE:
			if ((*car).speed < MIN_PLANE_SPEED)
				(*car).speed = MIN_PLANE_SPEED;
			break;
		default:
			break;
		}
		break;
	case BUTTON_REVERSE:
		/* reverse */
		(*car).direction = - (*car).direction;
		break;
	case BUTTON_COLOR:
		/* change car color */
		(*car).color = ((*car).color+1) % MAX_COLORS;
		break;
	case BUTTON_BIGGER:
		/* make car bigger */
		(*car).width = ((*car).width + 2);
		(*car).height = ((*car).height + 1);
		break;
	case BUTTON_SMALLER:
		/* make car smaller */
		(*car).width = ((*car).width - 2);
		if ((*car).width < MIN_VEHICLE_WIDTH) 
			(*car).width = MIN_VEHICLE_WIDTH;
		(*car).height = ((*car).height - 1);
		if ((*car).height < MIN_VEHICLE_HEIGHT) 
			(*car).height = MIN_VEHICLE_HEIGHT;
		break;
	case BUTTON_LANE_UP:
		/* change lanes up,		round to nearest 10s */
		(*car).y = (*car).y + HIGHWAY_WIDTH;
		if(		((*car).type == VEHICLE_CAR || 
				 (*car).type == VEHICLE_TRUCK)  &&
				((*car).y > HIGH_GROUND_Y)        )
			(*car).y = HIGH_GROUND_Y;
		if(		((*car).type == VEHICLE_PLANE || 
				 (*car).type == VEHICLE_CHOPPER)  &&
				((*car).y > HIGH_AIR_Y)        )
			(*car).y = HIGH_AIR_Y;
		break;
	case BUTTON_LANE_DOWN:
		/* change lanes down,	round to nearest 10s */
		(*car).y = (*car).y - HIGHWAY_WIDTH;
		if(		((*car).type == VEHICLE_CAR || 
				 (*car).type == VEHICLE_TRUCK)  &&
				((*car).y < LOW_GROUND_Y)        )
			(*car).y = LOW_GROUND_Y;
		if(		((*car).type == VEHICLE_PLANE || 
				 (*car).type == VEHICLE_CHOPPER)  &&
				((*car).y < LOW_AIR_Y)        )
			(*car).y = LOW_AIR_Y;
		break;
	case BUTTON_FIRE:
		/* fire a missile in front of vehicle */
		(*settings).p1firing = TRUE;
		break;
	case BUTTON_SELECT_NEXT:
		/* select the next car */
		(*car).player = PLAYER_NONE;
		*car_selected += 1;
		if (*car_selected == *car_selected2)
			*car_selected += 1;
		if (*car_selected > ((*settings).cars - 1))
			*car_selected = 0;
		(*car).player = PLAYER_1;
		break;
	case BUTTON_SELECT_LAST:
		/* select the previous car */
		(*car).player = PLAYER_NONE;
		*car_selected -= 1;
		if (*car_selected == *car_selected2)
			*car_selected -= 1;
		if (*car_selected < 0)
			*car_selected = ((*settings).cars - 1);
		(*car).player = PLAYER_1;
		break;
	case BUTTON_EXIST:
		/* toggle whether car exists or not */
		if ((*car).exists == TRUE) {
			(*car).exists = FALSE;
		} else {
			(*car).exists = TRUE;
		}
		break;
	case BUTTON_RANDOMIZE:
		/* create new data for the selected vehicle */
		randomize_vehicle(car);
		break;
	case BUTTON_PAUSE:
		/* Pause action, allow game settings mode */
		(*settings).paused = TRUE;
		pause_game();
		(*settings).reset = TRUE;
		break;
	default:
		/*Bad news if this is ever reached! */
		something_is_wrong();
		break;
	}
}


/* Process button click at location x, y.  Update game settings	*/
/* direction, color, and whether traffic is moving as needed.	*/
void handle_button_paused(int x, int y, game_data *settings)
{
	int buttonNum = -1;	/* number of selected button, -1 if none */
	
	/* Convert y coordinate to button number from top. */
	if (x < (MENU_LEFT_PAUSED + BUTTON_WIDTH_PAUSED) && 
			(x > MENU_LEFT_PAUSED) && (y < MENU_TOP_PAUSED) )
		buttonNum = (MENU_TOP_PAUSED - y) / BUTTON_HEIGHT_PAUSED + 1;
	/* recognize Resume Button clicks */
	if (x < (BUTTON_RESUME_LEFT + BUTTON_RESUME_WIDTH) &&
			(x > BUTTON_RESUME_LEFT) && (y > BUTTON_RESUME_TOP) &&
			(y < BUTTON_RESUME_TOP + BUTTON_RESUME_HEIGHT) )
		buttonNum = BUTTON_RESUME_PAUSED;
	
	switch(buttonNum) {
	case BUTTON_RESUME_PAUSED:
		/* unpause the game */
		(*settings).paused = FALSE;
		break;
	case BUTTON_MORE_CARS_PAUSED:
		/* use more cars */
		(*settings).cars += 1;
		if ((*settings).cars > MAX_DECLARABLE_VEHICLES) {
			(*settings).cars = MAX_DECLARABLE_VEHICLES;
			GP142_printfXY(RED, 0, 0, 16, "Maximum Vehicles Set");
		}
		(*settings).reset = TRUE;
		break;
	case BUTTON_LESS_CARS_PAUSED:
		/* use less cars */
		(*settings).cars -= 1;
		if ((*settings).cars < MIN_DECLARABLE_VEHICLES) {
			(*settings).cars = MIN_DECLARABLE_VEHICLES;
			GP142_printfXY(RED, 0, 0, 16, "Minimum Vehicles Set");
		}
		(*settings).reset = TRUE;
		break;
	case BUTTON_MAX_CARS_PAUSED:
		(*settings).cars = MAX_DECLARABLE_VEHICLES;
		(*settings).reset = TRUE;
		break;
	case BUTTON_MIN_CARS_PAUSED:
		(*settings).cars = MIN_DECLARABLE_VEHICLES;
		(*settings).reset = TRUE;
		break;
	case BUTTON_RESET_SCORES_PAUSED:
		(*settings).score1 = 0;
		(*settings).score2 = 0;
		(*settings).reset = TRUE;
		break;
	case BUTTON_FAST_DRAW:
		(*settings).fastdraw = TRUE;
		(*settings).reset = TRUE;
		break;
	case BUTTON_SLOW_DRAW:
		(*settings).fastdraw = TRUE;
		(*settings).reset = TRUE;
		break;

	default:
		break;
	}
}

/*****************************************************************************
     TRAFFIC AND OTHER ANIMATED OBJECTS
*****************************************************************************/


/* Update sun coordinates sun_x, sun_y by one tick */
void move_sun(int *sun_x, int *sun_y) {
	/* Move the sun to the left.  If it reaches the edge, move it */
	/* to the other side and down a bit. */
	/* This is assuming looking north from south of equator.  :)  */
	(*sun_x) --;
	if (*sun_x < -GP142_XMAX -30) {
		*sun_x = - *sun_x;
		(*sun_y) --;
	}
}


/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (EAST or WEST), and speed in pixels per tick, */
void move_traffic(int *x, int *y, int *direction, int speed) {
	
	int trial_x;		/* new value for x coordinate */
	
	/* move vehicle.  If it moves off the edge, reverse direction */
	trial_x = *x + (*direction * speed);
	
	if (trial_x > GP142_XMAX) {
		trial_x = -GP142_XMAX;  /* because 0,0 is center */
	} else {
		if (trial_x < -GP142_XMAX) {
			trial_x = GP142_XMAX;
		}
	}
	
	*x = trial_x;
}


/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int width, int height, int color) {
	GP142_rectangleXY(color, 
			x - width / 2,  y + height / 4,
			x + width / 2,  y + height * 3/4, FILL);
	GP142_rectangleXY(color, 
			x - width / 5,  y + height * 3/4,
			x + width / 5,  y + height, FILL);
	GP142_triangleXY(color,
			x - width * 3/10,  y + height * 3/4,
			x - width / 5,  y + height * 3/4,
			x - width / 5,  y + height,  FILL);
	GP142_triangleXY(color,
			x + width * 3/10,  y + height * 3/4,
			x + width / 5,  y + height * 3/4,
			x + width / 5,  y + height,  FILL);
	GP142_circleXY(BLACK, x - width / 4,  y + height / 4,  height / 4);
	GP142_circleXY(BLACK, x + width / 4,  y + height / 4,  height / 4);
}

/* Draw a single truck of the given color with horizontal */
/* center x and bottom y*/
void draw_truck(int x, int y, int width, int height, int color, int dir) {
	GP142_rectangleXY(color, 
			x - width / 2,  y + height / 4,
			x + width / 2,  y + height * 2/3, FILL);
	GP142_circleXY(BLACK, x - width / 4,  y + height / 4,  height / 4);
	GP142_circleXY(BLACK, x + width / 4,  y + height / 4,  height / 4);
	if (dir == WEST) {
		GP142_rectangleXY(color, 
				x - width / 4,  y + height * 2/3,
				x,			 y + height,  FILL);
		GP142_triangleXY(color,
				x - width * 3/8,  y + height * 2/3,
				x - width / 4,  y + height * 2/3,
				x - width / 4,  y + height,  FILL);
	} else {
		GP142_rectangleXY(color, 
				x + width / 4,  y + height * 2/3,
				x,			 y + height,  FILL);
		GP142_triangleXY(color,
				x + width * 3/8,  y + height * 2/3,
				x + width / 4,  y + height * 2/3,
				x + width / 4,  y + height,  FILL);
	}
}

/* Draw a single plane of the given color with horizontal */
/* center x and bottom y*/
void draw_plane(int x, int y, int width, int height, int color, int dir) {
	if (dir == WEST) {
		GP142_rectangleXY(color, 
				x - width * 3/10,  y + height * 5/12,
				x + width / 2,  y + height * 3/4, FILL);
		GP142_triangleXY(color,
				x - width / 2,  y + height * 5/12,
				x - width * 3/10,  y + height * 5/12,
				x - width * 3/10,  y + height * 3/4,  FILL);
		GP142_triangleXY(WHITE,
				x - width * 3/20,  y + height / 2,
				x + width * 3/20,  y + height / 2,
				x + width * 3/20,  y,  FILL);
		GP142_triangleXY(WHITE,
				x + width * 7/20,  y + height * 3/4,
				x + width / 2,  y + height * 3/4,
				x + width / 2,  y + height,  FILL);
	} else {
		GP142_rectangleXY(color, 
				x + width * 3/10,  y + height * 5/12,
				x - width / 2,  y + height * 3/4, FILL);
		GP142_triangleXY(color,
				x + width / 2,  y + height * 5/12,
				x + width * 3/10,  y + height * 5/12,
				x + width * 3/10,  y + height * 3/4,  FILL);
		GP142_triangleXY(WHITE,
				x + width * 3/20,  y + height / 2,
				x - width * 3/20,  y + height / 2,
				x - width * 3/20,  y,  FILL);
		GP142_triangleXY(WHITE,
				x - width * 7/20,  y + height * 3/4,
				x - width / 2,  y + height * 3/4,
				x - width / 2,  y + height,  FILL);
	}
}

/* Draw a single chopper of the given color with horizontal */
/* center x and bottom y*/
void draw_chopper(int x, int y, int width, int height, int color, int dir) {
	if (dir == WEST) {
		GP142_rectangleXY(BLACK, 
				x - width * 5/40,  y + height * 3/4,
				x - width * 7/70,  y + height,  FILL);
		GP142_rectangleXY(BLACK, 
				x - width / 2,  y + height * 11/12,
				x + width / 5,  y + height,  FILL);
		GP142_rectangleXY(color, 
				x,			 y + height / 2,
				x + width / 2,  y + height / 3,  FILL);
		GP142_circleXY(color, x - width * 3/20,  y + height * 5/12,  height * 5/12);
		GP142_triangleXY(WHITE,
				x + width * 2/5,  y + height / 2,
				x + width / 2,	   y + height / 2,
				x + width / 2,    y + height * 3/4,  FILL);
	} else {
		GP142_rectangleXY(BLACK, 
				x + width * 5/40,  y + height * 3/4,
				x + width * 7/70,  y + height,  FILL);
		GP142_rectangleXY(BLACK, 
				x + width / 2,  y + height * 11/12,
				x - width / 5,  y + height,  FILL);
		GP142_rectangleXY(color, 
				x,			 y + height / 2,
				x - width / 2,  y + height / 3,  FILL);
		GP142_circleXY(color, x + width * 3/20,  y + height * 5/12,  height * 5/12);
		GP142_triangleXY(WHITE,
				x - width * 2/5,  y + height / 2,
				x - width / 2,	   y + height / 2,
				x - width / 2,    y + height * 3/4,  FILL);
	}
}

/* Draw a single explosion with horizontal center x and bottom y*/
void draw_explosion(int x, int y, int width, int height) {
	GP142_circleXY(RED, x - width / 2,  y + height / 2,  height / 2);
	/* <<< MAKE MORE DRAMATIC, MAYBE ADD SOUND LATER? >>> */
	
}

/* Draw a single missile with horizontal center x and bottom y*/
void draw_missile(int x, int y, int width, int height, int dir) {
	y = y + 4;
	/* <<< MAKE BETTER LATER >>> */
	GP142_rectangleXY(BLACK, x, y, x + MISSILE_WIDTH, y + MISSILE_HEIGHT, FILL);
	
}


void draw_button_car(int type, int width, int height, int color, int dir, int player) {
	if (player == PLAYER_1) {
		switch (type) {
		case VEHICLE_CAR:
			draw_car(		BUTTON_CAR_X, BUTTON_CAR_Y, width, height, color);
			break;
		case	VEHICLE_TRUCK:
			draw_truck(	BUTTON_CAR_X, BUTTON_CAR_Y, width, height, color, dir);
			break;
		case VEHICLE_PLANE:
			draw_plane(	BUTTON_CAR_X, BUTTON_CAR_Y, width, height, color, dir);
			break;
		case VEHICLE_CHOPPER:
			draw_chopper(	BUTTON_CAR_X, BUTTON_CAR_Y, width, height, color, dir);
			break;
		case VEHICLE_EXPLOSION:
			draw_explosion(BUTTON_CAR_X, BUTTON_CAR_Y, width, height);
			break;
		case VEHICLE_MISSILE:
			draw_missile(BUTTON_CAR_X, BUTTON_CAR_Y, width, height, dir);
			break;
		default:
			something_is_wrong();
			break;
		}
	} else {
		switch (type) {
		case VEHICLE_CAR:
			draw_car(		BUTTON_CAR_X2, BUTTON_CAR_Y2, width, height, color);
			break;
		case	VEHICLE_TRUCK:
			draw_truck(	BUTTON_CAR_X2, BUTTON_CAR_Y2, width, height, color, dir);
			break;
		case VEHICLE_PLANE:
			draw_plane(	BUTTON_CAR_X2, BUTTON_CAR_Y2, width, height, color, dir);
			break;
		case VEHICLE_CHOPPER:
			draw_chopper(	BUTTON_CAR_X2, BUTTON_CAR_Y2, width, height, color, dir);
			break;
		case VEHICLE_EXPLOSION:
			draw_explosion(BUTTON_CAR_X2, BUTTON_CAR_Y2, width, height);
			break;
		case VEHICLE_MISSILE:
			draw_missile(BUTTON_CAR_X2, BUTTON_CAR_Y2, width, height, dir);
			break;
		default:
			something_is_wrong();
			break;
		}
	}
}


void randomize_vehicle(vehicle *car) {
	int i = 0;		/* temp vars */
	
	i = rand();  /* by default, rand() can be huge int */
	/* rand == even == 50% chance */
	if ((double) i / 2.0 == (int) i / 2)  
		(*car).direction = EAST;
	else (*car).direction = WEST;
	/* recreate type may NOT be an special vehicle (last type) */
	(*car).type = rand() % (NUMBER_VEHICLE_TYPES - SPECIAL_VEHICLES);
	(*car).exists = TRUE;
	(*car).moving = TRUE;
	(*car).x = (rand() % 1001) - 500;	/* -500 to 500	*/

	switch ((*car).type) {
	case VEHICLE_CAR:
		(*car).y = HIGHWAY_CENTER - 
				(rand() % NUMBER_LANES) * HIGHWAY_WIDTH;
		/* compensate to avoid placing on meridian strip */
		if ((*car).y < (HIGHWAY_CENTER - 2 * HIGHWAY_WIDTH))
			(*car).y -= HIGHWAY_WIDTH;
		(*car).speed = (rand() % RAND_CAR_RANGE) 
				+ TRUCK_MIN_SPEED;
		(*car).width = CAR_START_WIDTH;
		(*car).height = CAR_START_HEIGHT;
		break;
	case VEHICLE_TRUCK:
		(*car).y = HIGHWAY_CENTER - 
				(rand() % NUMBER_LANES) * HIGHWAY_WIDTH;
		/* compensate to avoid placing on meridian strip */
		if ((*car).y < (HIGHWAY_CENTER - 2 * HIGHWAY_WIDTH))
			(*car).y -= HIGHWAY_WIDTH;
		(*car).speed = (rand() % RAND_TRUCK_RANGE) 
				+ TRUCK_MIN_SPEED;
		(*car).width = TRUCK_START_WIDTH;
		(*car).height = TRUCK_START_HEIGHT;
		break;
	case VEHICLE_PLANE:
		/* For now, anywhere on top half of screen will work */
		(*car).y = (rand() % (GP142_YMAX / 10)) * 10;
		(*car).speed = (rand() % RAND_PLANE_RANGE) 
				+ PLANE_MIN_SPEED;
		(*car).width = PLANE_START_WIDTH;
		(*car).height = PLANE_START_HEIGHT;
		break;
	case VEHICLE_CHOPPER:
		/* For now, anywhere on top half of screen will work */
		(*car).y = (rand() % (GP142_YMAX / 10)) * 10;
		(*car).speed = (rand() % RAND_CHOPPER_RANGE) 
				+ CHOPPER_MIN_SPEED;
		(*car).width = CHOPPER_START_WIDTH;
		(*car).height = CHOPPER_START_HEIGHT;
		break;
	} /* end switch for (*car).type */
	
	(*car).frame = 0;
	(*car).color = rand() % MAX_COLORS;
	
}


/* Display the 2 players' scores */
void draw_scores(int score1, int score2) {
	/* <<< Dont know how to fix GP142_printfXY from printing garbledygook  */
	/*     so using newlines to slide it off screen.  Maybe fix later? >>> */
	char digits1[19] = {'P', '1', ' ', 'S', 'c', 'o', 'r', 'e', ':', 
			'\n', '0', '0', '0', '0', '0', '\n', '\n', '\n', '\n'   };
	char digits2[19] = {'P', '2', ' ', 'S', 'c', 'o', 'r', 'e', ':', 
			'\n', '0', '0', '0', '0', '0', '\n', '\n', '\n', '\n'   };
	
	/* turn player 1 score into characters */
	digits1[14] = (score1 % 10);
	digits1[13] = (score1 % 100 - digits1[14]) / 10;
	digits1[12] = (score1 % 1000 - digits1[13]*10 - digits1[14]) / 100;
	digits1[11] = (score1 % 10000 - digits1[12]*100 - digits1[13]*10 
			- digits1[14]) / 1000;
	digits1[10] = (score1 % 100000 - digits1[11]*1000 - digits1[12]*100 
			- digits1[13]*10 - digits1[14]) / 10000;
	/* using bad numbers as characters, fix now. */
	digits1[14] += ASCII_DEC_MOD;
	digits1[13] += ASCII_DEC_MOD;
	digits1[12] += ASCII_DEC_MOD;
	digits1[11] += ASCII_DEC_MOD;
	digits1[10] += ASCII_DEC_MOD;

	/* turn player 3 score into characters */
	digits2[14] = (score2 % 10);
	digits2[13] = (score2 % 100 - digits2[14]) / 10;
	digits2[12] = (score2 % 1000 - digits2[13]*10 - digits2[14]) / 100;
	digits2[11] = (score2 % 10000 - digits2[12]*100 - digits2[13]*10 
			- digits2[14]) / 1000;
	digits2[10] = (score2 % 100000 - digits2[11]*1000 - digits2[12]*100 
			- digits2[13]*10 - digits2[14]) / 10000;
	/* using bad numbers as characters, fix now. */
	digits2[14] += ASCII_DEC_MOD;
	digits2[13] += ASCII_DEC_MOD;
	digits2[12] += ASCII_DEC_MOD;
	digits2[11] += ASCII_DEC_MOD;
	digits2[10] += ASCII_DEC_MOD;

	GP142_printfXY(BLUE, -GP142_XMAX + 15, -GP142_YMAX + 30, 18, digits1);
	GP142_printfXY(BLUE, GP142_XMAX - 95, -GP142_YMAX + 30, 18, digits2);
	
	
}


/* Setup graphics for paused game loop */
void pause_game(void) {
	GP142_printfXY(BLACK, -GP142_XMAX + 65, GP142_YMAX - 105, 72, 
			"GAME PAUSED");
	draw_buttons_paused();
}


/*****************************************************************************
     OTHER
*****************************************************************************/


/* Initialize random number generator */
/* [Ignore or delete if your program  */
/* doesn't user random numbers.] */
void initialize_random_numbers() {
	time_t t;			/* current time */
	srand (time(&t));
}



/* Display error message if internal error is detected. */
void something_is_wrong (void) {
	GP142_printfXY (RED, 0, 0, 16, "BAAAADDD NEWS!");
}